/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* rc-debug-misc.c
 * Copyright (C) 2000, 2001 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include <stdio.h>

#include "rc-debug.h"
#include "rc-debug-misc.h"

static void
debug_dump_dephash_iterator (gpointer key, gpointer val, gpointer data)
{
    RCPackageDep *hpdep = (RCPackageDep *) key;
    RCPackage *hppkg = (RCPackage *) val;

    rc_debug (RC_DEBUG_LEVEL_INFO, "PACKAGE %s: ", hppkg->spec.name);
    debug_rc_package_dep_dump (RC_DEBUG_LEVEL_DEBUG, hpdep);
    debug_rc_package_dump (RC_DEBUG_LEVEL_DEBUG, hppkg);
}

void
debug_dump_dependency_hash (GHashTable *ht)
{
    g_hash_table_foreach (ht, debug_dump_dephash_iterator, NULL);
}

void
debug_rc_package_dep_slist_dump (int dlev, RCPackageDepSList *dep)
{
    while (dep) {
        debug_rc_package_dep_dump (dlev, (RCPackageDep *) dep->data);
        dep = dep->next;
    }
    rc_debug (dlev, "\n");
}

gchar *
debug_rc_package_dep_slist_str (RCPackageDepSList *depsl)
{
    gchar *s = g_strdup("");
    gchar *a, *b;
    while (depsl) {
        b = debug_rc_package_dep_str ((RCPackageDep *) depsl->data);
        a = g_strconcat (s, ", ", b, NULL);
        g_free (s);
        g_free (b);
        s = a;
        depsl = depsl->next;
    }

    return s;
}

void
debug_rc_package_dep_dump (int dlev, RCPackageDep *di)
{
    gchar *s = debug_rc_package_dep_str (di);
    rc_debug (dlev, "%s\n", s);
    g_free (s);
}

gchar *
debug_rc_package_dep_str (RCPackageDep *di)
{
    gchar *s;
    gchar *r;
    s = rc_package_spec_to_str (&di->spec);
    r = g_strdup_printf ("[%s%s %s]",
                         di->relation & RC_RELATION_WEAK ? "W" : " ",
                         rc_package_relation_to_string (di->relation, FALSE),
                         s);
    g_free (s);
    return r;
}

void
debug_rc_package_dump (int dlev, RCPackage *hpd)
{
    GSList *gl;

    if (!hpd) {
        rc_debug (dlev, "data: (null)\n");
    } else {
        rc_debug (dlev, "data: nm: %s vs: %d:%s rl: %s\n",
                 hpd->spec.name, hpd->spec.epoch, hpd->spec.version, hpd->spec.release);
        gl = hpd->requires;
        if (gl) rc_debug (dlev, "requires:\n");
        while (gl) {
            rc_debug (dlev, "   ");
            debug_rc_package_dep_dump (dlev, (RCPackageDep *) gl->data);
            gl = gl->next;
        }
        gl = hpd->provides;
        if (gl) rc_debug (dlev, "provides:\n");
        while (gl) {
            rc_debug (dlev, "   ");
            debug_rc_package_dep_dump (dlev, (RCPackageDep *) gl->data);
            gl = gl->next;
        }
        gl = hpd->conflicts;
        if (gl) rc_debug (dlev, "conflicts:\n");
        while (gl) {
            rc_debug (dlev, "   ");
            debug_rc_package_dep_dump (dlev, (RCPackageDep *) gl->data);
            gl = gl->next;
        }
    }
}

#if 0
void
debug_dump_name_pkg_iterator (gpointer name, gpointer pkg, gpointer arg)
{
    gchar *nm = (gchar *) name;
    RCPackage *pn = (RCPackage *) pkg;
    gchar *pnm;

    pnm = rc_package_spec_to_str (&pn->spec);

    rc_debug (RC_DEBUG_LEVEL_DEBUG, "%s: %s\n", nm, pnm);
    g_free (pnm);
}
#endif

void
debug_rc_package_slist_dump (int dlev, RCPackageSList *pkgs)
{
    while (pkgs) {
        RCPackage *pkg = (RCPackage *) pkgs->data;

        rc_debug (dlev, "%s (%d:%s-%s), ", pkg->spec.name, pkg->spec.epoch, pkg->spec.version,
                  pkg->spec.release);

        pkgs = pkgs->next;
    }
}
