/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * gui-log.c: Itemized logger
 *
 * Copyright (C) 2000, 2001 Ximian, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gnome.h>
#include "gui.h"

static GtkObjectClass *gui_log_parent;

enum SIGNALS {
    APPENDED,
    RESET,
    LAST_SIGNAL
};

static guint signals[LAST_SIGNAL] = { 0 };

struct _GUILogPriv {
    GSList *log;
};

static void
gui_log_destroy(GtkObject *obj)
{
    GUILog *log = GUI_LOG(obj);

    g_return_if_fail(log->priv);

    g_slist_foreach(log->priv->log, (GFunc) g_free, NULL);
    g_slist_free(log->priv->log);
    log->priv->log = NULL;

    g_free(log->priv);
    log->priv = NULL;

    if (GTK_OBJECT_CLASS(gui_log_parent)->destroy)
        (* GTK_OBJECT_CLASS(gui_log_parent)->destroy)(obj);
} /* gui_log_destroy */

static void
gui_log_class_init(GUILogClass *klass)
{
    GtkObjectClass *object_class = (GtkObjectClass *) klass;

    object_class->destroy = gui_log_destroy;

    gui_log_parent = gtk_type_class(gtk_object_get_type());

    signals[APPENDED] =
        gtk_signal_new(
            "appended",
            GTK_RUN_LAST,
            object_class->type,
            GTK_SIGNAL_OFFSET(GUILogClass, appended),
            gtk_marshal_NONE__POINTER,
            GTK_TYPE_NONE, 1,
            GTK_TYPE_POINTER);

    signals[RESET] =
        gtk_signal_new(
            "reset",
            GTK_RUN_LAST,
            object_class->type,
            GTK_SIGNAL_OFFSET(GUILogClass, reset),
            gtk_marshal_NONE__NONE,
            GTK_TYPE_NONE, 0);

    gtk_object_class_add_signals(object_class, signals, LAST_SIGNAL);
} /* gui_log_class_init */

static void
gui_log_init(GUILog *log)
{
    log->priv = g_new0(GUILogPriv, 1);
    log->priv->log = NULL;
} /* gui_log_init */

guint
gui_log_get_type(void)
{
    static guint type = 0;

    if (!type) {
        GtkTypeInfo type_info = {
            "GUILog",
            sizeof(GUILog),
            sizeof(GUILogClass),
            (GtkClassInitFunc) gui_log_class_init,
            (GtkObjectInitFunc) gui_log_init,
            (GtkArgSetFunc) NULL,
            (GtkArgGetFunc) NULL
        };

        type = gtk_type_unique(gtk_object_get_type(), &type_info);
    }

    return type;
} /* gui_log_get_type */

GUILog *
gui_log_new(void)
{
    GUILog *log;

    log = GUI_LOG(gtk_type_new(gui_log_get_type()));

    return log;
} /* gui_log_new */

void
gui_log_reset(GUILog *log)
{
    g_return_if_fail(log);
    g_return_if_fail(log->priv);

    g_slist_foreach(log->priv->log, (GFunc) g_free, NULL);
    g_slist_free(log->priv->log);
    log->priv->log = NULL;

    gtk_signal_emit(GTK_OBJECT(log), signals[RESET]);
} /* gui_log_reset */

void
gui_log_append(GUILog *log, char *message)
{
    char *m;

    g_return_if_fail(log);
    g_return_if_fail(log->priv);
    g_return_if_fail(message);

    m = g_strdup(message);

    log->priv->log = g_slist_append(log->priv->log, m);
    gtk_signal_emit(GTK_OBJECT(log), signals[APPENDED], (const char *) m);
} /* gui_log_append */

GSList *
gui_log_get(GUILog *log)
{
    g_return_val_if_fail(log, NULL);
    g_return_val_if_fail(log->priv, NULL);

    return log->priv->log;
} /* gui_log_get */

char *
gui_log_to_string(GUILog *log)
{
    GSList *i;
    char *out = NULL;

    g_return_val_if_fail(log, NULL);
    g_return_val_if_fail(log->priv, NULL);

    i = log->priv->log;
    while (i) {
        char *this = (char *) i->data;
        char *o;

        if (out) {
            o = g_strconcat(out, "\n", this, NULL);
            g_free(out);
            out = o;
        }
        else
            out = g_strdup(this);

        i = i->next;
    }

    return out;
} /* gui_log_to_string */
